/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#ifndef __COMMON_H__
#define __COMMON_H__

#include <stdint.h>
#include <stdarg.h>

#include "gus.h"
#include "gf1hware.h"


#include <stdlib.h>   // getenv
#include <string.h>   // memset
#include <stdio.h>    // sscanf

#include <dos.h>
#include <conio.h>

#define ENTER_CRITICAL     __asm { "pushf\n cli\n" }
#define LEAVE_CRITICAL     __asm { "popf\n" }
#define outportb outp
#define outportw outpw
#define inportb inp
#define inportw inpw

typedef struct _st_ultra_data
{
	volatile uint32_t flags;

#define DRAM_DMA_BUSY   0x0002   // show this channels dram dma is busy
#define ADC_DMA_BUSY    0x0004   // show we are busy recording
#define DRAM_DMA_NOWAIT 0x0008   // show we didn't wait last time
#define ADC_DMA_NOWAIT  0x0010   // show we didn't wait last time

	int      base_port;
	int      dram_dma_chan;
	int      adc_dma_chan;
	int      gf1_irq_num;
	int      midi_irq_num;
	MIDI_XMIT_CB midi_xmit_func;
	MIDI_RECV_CB midi_recv_func;
	GUS_TMR_CB timer1_func;
	GUS_TMR_CB timer2_func;
	GUS_WAV_CB wavetable_func;
	GUS_VOL_CB volume_func;
	GUS_DMA_CB dram_dma_tc_func;
	GUS_REC_CB record_dma_tc_func;
	GUS_AUX_CB aux_irq_func;
	uint8_t  mix_image;
	uint8_t  voices;
	uint8_t  image_midi;
	uint8_t  padding;
	uint32_t used_voices;
	uint32_t reserved_dram;
	uint32_t free_mem;
	uint8_t  timer_ctrl;
	uint8_t  timer_mask;
	int      midi_data;
	int      midi_control;
	int      voice_select;
	int      reg_select;
	int      data_low;
	int      data_hi;
	int      irq_status;
	int      dram_data;
	int      mix_control;
	int      irq_control;
	int      timer_control;
	int      timer_data;
	int      ultra_errno;
	int      gf1_sema4;
	int      irq_pending;

} ULTRA_DATA;


#define USE_ROLLOVER	0x01

extern ULTRA_DATA _gf1_data;

extern void gf1_delay (void);
extern uint32_t make_physical_address (uint32_t low, uint32_t high, uint8_t mode);
extern uint32_t convert_to_16bit (uint32_t address);

extern void ReSetIrqHandlers (int gf1_irq, int midi_irq);
extern void SetIrqHandlers (int gf1_irq, int midi_irq);
extern void SetIrqs (int gf1_irq, int midi_irq);
extern void ResetIrqs (int gf1_irq, int midi_irq);
extern uint8_t get_irq_latch (int channel);

extern void handle_dma_tc (void);
extern uint8_t get_dma_latch (int channel);

#endif
