/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#include "common.h"

#define READ_DMA		1
#define WRITE_DMA		2
#define INDEF_READ		3	/* auto init record */
#define INDEF_WRITE		4	/* auto init record */

#define DMA_AUTO_INIT		0x01		/* Used by app to specify autoinit */

#define DMA_16		0x40
#define DMA_8		0x00

#define DMA_CVT_2	0x80
#define DMA_NO_CVT	0x00



// DMA Controler #1 (8-bit controller)
#define DMA1_STAT     0x08          // read status register
#define DMA1_WCMD     0x08          // write command register
#define DMA1_WREQ     0x09          // write request register
#define DMA1_SNGL     0x0A          // write single bit register
#define DMA1_MODE     0x0B          // write mode register
#define DMA1_CLRFF    0x0C          // clear byte ptr flip/flop
#define DMA1_MCLR     0x0D          // master clear register
#define DMA1_CLRM     0x0E          // clear mask register
#define DMA1_WRTALL   0x0F          // write all mask register

// DMA Controler #2 (16-bit controller)
#define DMA2_STAT     0xD0          // read status register
#define DMA2_WCMD     0xD0          // write command register
#define DMA2_WREQ     0xD2          // write request register
#define DMA2_SNGL     0xD4          // write single bit register
#define DMA2_MODE     0xD6          // write mode register
#define DMA2_CLRFF    0xD8          // clear byte ptr flip/flop
#define DMA2_MCLR     0xDA          // master clear register
#define DMA2_CLRM     0xDC          // clear mask register
#define DMA2_WRTALL   0xDE          // write all mask register

#define DMA0_ADDR     0x00          // chan 0 base adddress
#define DMA0_CNT      0x01          // chan 0 base count
#define DMA1_ADDR     0x02          // chan 1 base adddress
#define DMA1_CNT      0x03          // chan 1 base count
#define DMA2_ADDR     0x04          // chan 2 base adddress
#define DMA2_CNT      0x05          // chan 2 base count
#define DMA3_ADDR     0x06          // chan 3 base adddress
#define DMA3_CNT      0x07          // chan 3 base count
#define DMA4_ADDR     0xC0          // chan 4 base adddress
#define DMA4_CNT      0xC2          // chan 4 base count
#define DMA5_ADDR     0xC4          // chan 5 base adddress
#define DMA5_CNT      0xC6          // chan 5 base count
#define DMA6_ADDR     0xC8          // chan 6 base adddress
#define DMA6_CNT      0xCA          // chan 6 base count
#define DMA7_ADDR     0xCC          // chan 7 base adddress
#define DMA7_CNT      0xCE          // chan 7 base count

#define DMA0_PAGE     0x87          // chan 0 page register (refresh)*/
#define DMA1_PAGE     0x83          // chan 1 page register
#define DMA2_PAGE     0x81          // chan 2 page register
#define DMA3_PAGE     0x82          // chan 3 page register
#define DMA4_PAGE     0x8F          // chan 4 page register (unuseable)*/
#define DMA5_PAGE     0x8B          // chan 5 page register
#define DMA6_PAGE     0x89          // chan 6 page register
#define DMA7_PAGE     0x8A          // chan 7 page register


#define MAX_DMA           7

#define DMA_DECREMENT     0x20     // mask to make DMA hardware go backwards

// Bits for dma flags location

#define DMA_USED     0x0001
#define DMA_PENDING  0x0002
#define TWO_FLAG     0x0004
#define REV_FLAG     0x0008
#define CALIB_COUNT  0x0010

typedef struct _st_dma_entry
{
	uint32_t flags;
	uint32_t latch;
	uint32_t dma_disable;       // bits to disable dma channel
	uint32_t dma_enable;        // bits to enable dma channel
	uint32_t page;              // page port location
	uint32_t addr;              // addr port location
	uint32_t count;             // count port location
	uint32_t single;            // single mode port location
	uint32_t mode;              // mode port location
	uint32_t clear_ff;          // clear flip-flop port location
	uint32_t write;             // bits for write transfer
	uint32_t read;              // bits for read transfer
	uint8_t  cur_mode;          // current mode
	uint8_t  reserved;
	uint32_t cur_page;          // current page of transfer
	uint32_t cur_addr;          // current transfer address
	uint32_t amnt_sent;         // current amnt sent
	uint32_t cur_size;          // current size of transfer
	uint32_t nxt_page;          // next page
	uint32_t nxt_addr;          // next address
	uint32_t nxt_size;          // size of next buffer
	uint8_t  cur_control;

} DMA_ENTRY;


static DMA_ENTRY _gf1_dma[MAX_DMA] = {

/* DMA channel 1 */     { 0, 1, 0x05, 0x01, DMA1_PAGE, DMA1_ADDR, DMA1_CNT,
			  DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x49, 0x45 },

/* DMA channel 2 */     { 0, 0, 0x06, 0x02, DMA2_PAGE, DMA2_ADDR, DMA2_CNT,
			  DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x4A, 0x46 },

/* DMA channel 3 */     { 0, 2, 0x07, 0x03, DMA3_PAGE, DMA3_ADDR, DMA3_CNT,
			  DMA1_SNGL, DMA1_MODE, DMA1_CLRFF, 0x4B, 0x47 },

/* DMA channel 4 */     { 0, 0, 0x04, 0x00, DMA4_PAGE, DMA4_ADDR, DMA4_CNT,
			  DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x48, 0x44 },

/* DMA channel 5 */     { 0, 3, 0x05, 0x01, DMA5_PAGE, DMA5_ADDR, DMA5_CNT,
			  DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x49, 0x45 },

/* DMA channel 6 */     { 0, 4, 0x06, 0x02, DMA6_PAGE, DMA6_ADDR, DMA6_CNT,
			  DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x4A, 0x46 },

/* DMA channel 7 */     { 0, 5, 0x07, 0x03, DMA7_PAGE, DMA7_ADDR, DMA7_CNT,
			  DMA2_SNGL, DMA2_MODE, DMA2_CLRFF, 0x4B, 0x47 },
};


static void UltraDmaNext (DMA_ENTRY *tdma, int record)
{
	uint32_t tcount;

	tdma->flags &= ~TWO_FLAG;                 /* shut off rollover        */
	tdma->amnt_sent = tdma->cur_size;         /* save the amount xferred  */
	tdma->cur_size = tdma->nxt_size;          /* set up for next buffer   */

	tcount = tdma->nxt_size;// - 1;
	outportb (tdma->single, tdma->dma_disable);          /* disable chan   */
	outportb (tdma->clear_ff, 0);                        /* clear f/f      */
	outportb (tdma->addr, tdma->nxt_addr & 0xff);        /* LSB            */
	outportb (tdma->addr, (tdma->nxt_addr >> 8) & 0xff); /* MSB            */
	outportb (tdma->page, tdma->nxt_page);               /* page #         */
	outportb (tdma->mode, tdma->cur_mode);               /* set mode       */
	outportb (tdma->clear_ff, 0);                        /* clear f/f      */
	outportb (tdma->count, tcount & 0x0ff);              /* LSB count      */
	outportb (tdma->count, (tcount >> 8) & 0x0ff);       /* MSB count      */
	outportb (tdma->single, tdma->dma_enable);           /* enable         */

	if (record)
		UltraStartRecordDma (tdma->cur_control);
	else
		UltraStartDramDma (tdma->cur_control);
}


void handle_dma_tc (void)
{
	DMA_ENTRY *tdma;
	uint8_t val;

	/*
	 * First, check to see if we need to service DRAM DMA terminal
	 * count.  This read also clears the pending irq.
	 */

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, DMA_CONTROL);
	val = inportb (_gf1_data.data_hi);

	LEAVE_CRITICAL;

	if (val & DMA_IRQ_PENDING)
	{
		tdma = &_gf1_dma[_gf1_data.dram_dma_chan - 1];

		if (tdma->flags & TWO_FLAG)
		{
			// handle cross-over page

			UltraDmaNext (tdma, 0);
		}
		else
		{
			tdma->flags &= ~DMA_PENDING;

			_gf1_data.flags &= ~DRAM_DMA_BUSY;
			tdma->amnt_sent += tdma->cur_size;  // accumulate totals

			if (_gf1_data.dram_dma_tc_func)
				_gf1_data.dram_dma_tc_func ();
		}
	}


	/*
	 * Now check recording's terminal count
	 * This read also clears the pending irq.
	 */

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, SAMPLE_CONTROL);
	val = inportb (_gf1_data.data_hi);

	LEAVE_CRITICAL;

	if (val & ADC_IRQ_PENDING)
	{
		tdma = &_gf1_dma[_gf1_data.adc_dma_chan - 1];

		if (tdma->flags & TWO_FLAG)
		{
			// handle cross-over page

			UltraDmaNext (tdma, 1);
		}
		else
		{
			tdma->flags &= ~DMA_PENDING;

			_gf1_data.flags &= ~ADC_DMA_BUSY;
			tdma->amnt_sent += tdma->cur_size;  // accumulate totals

			if (_gf1_data.record_dma_tc_func)
				_gf1_data.record_dma_tc_func ();
		}
	}

	return;
}


uint8_t get_dma_latch (int channel)
{
	return _gf1_dma[channel - 1].latch;
}


/*
 * This function will start up a DMA transfer to the UltraSound
 */
void UltraStartDramDma (uint8_t control)
{
	control &= DMA_READ | DMA_16 | DMA_CVT_2;
	control |= DMA_R0;   // fast
	control |= DMA_ENABLE | DMA_IRQ_ENABLE;

	if (_gf1_data.dram_dma_chan >= 4)
		control |= DMA_WIDTH_16;   // 16-bit

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, DMA_CONTROL);
	outportb (_gf1_data.data_hi, control);

	LEAVE_CRITICAL;
}


void UltraStartRecordDma (uint8_t control)
{
	control &= ADC_MODE | ADC_TWOS_COMP;
	control |= ENABLE_ADC | ADC_IRQ_ENABLE;

	if (_gf1_data.adc_dma_chan >= 4)
		control |= ADC_DMA_WIDTH;   // 16-bit

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, SAMPLE_CONTROL);
	outportb (_gf1_data.data_hi, control);

	LEAVE_CRITICAL;
}


int PrimeDma(void *pc_ptr, int type, uint32_t size, int channel)
{
	DMA_ENTRY *tdma;
	uint32_t s_20bit, e_20bit;
	uint32_t spage, saddr, tcount;
	uint32_t epage, eaddr;

	tdma = &_gf1_dma[channel - 1];

	if (tdma->flags & DMA_PENDING)
		return(DMA_BUSY);

	tdma->flags |= DMA_PENDING;
	tdma->flags |= CALIB_COUNT;

	/*
	 * Convert the pc address to a 20 bit physical address
	 * that the DMA controller needs
	 */
	s_20bit = ((unsigned long) pc_ptr) & 0x000fffff;
	e_20bit = s_20bit + size - 1;

	spage = (s_20bit & 0xffff0000L) >> 16;
	epage = (e_20bit & 0xffff0000L) >> 16;

	if (channel >= 4)
	{
		// if 16-bit xfer, then addr,count & size are divided by 2
		s_20bit = s_20bit >> 1;
		e_20bit = e_20bit >> 1;
		size = size >> 1;
	}

	saddr = s_20bit & 0x0000ffffL;
	eaddr = e_20bit & 0x0000ffffL;

	/*
	 * In case the buffer goes over a page, save the data
	 * for the irq handler to use to finish sending the data
	 */

	if (spage != epage)
	{
		tdma->flags |= TWO_FLAG;
		tdma->nxt_page = epage;
		if (channel >= 4)
		{
			if (tdma->nxt_page & 0x01)
				tdma->nxt_addr = 0x8000;
			else
				tdma->nxt_addr = 0;
			eaddr &= 0x7fff;
		}
		else
			tdma->nxt_addr = 0;

		tdma->nxt_size = eaddr;
		size -= eaddr - 1;   // only supposed to send this much
	}
	else
		tdma->flags &= ~TWO_FLAG;

	if ((type == INDEF_READ) && (tdma->flags & TWO_FLAG))
		return BAD_DMA_ADDR;

	tdma->cur_page  = spage;
	tdma->cur_addr  = saddr;
	tdma->amnt_sent = 0;          // init the amount xferred so far
	tdma->cur_size  = size;       // show how big THIS part of buffer is

	tcount = size - 1;

	switch (type)
	{
	case READ_DMA:
		tdma->cur_mode = tdma->read;
		break;
	case WRITE_DMA:
		tdma->cur_mode = tdma->write;
		break;
	case INDEF_READ:
		/*
		 * with this mode, all we have to do is kick off the gf1
		 * sample control to restart the dma recording ....
		 */
		tdma->cur_mode = tdma->read | 0x10;   // turn on auto init
		break;
	case INDEF_WRITE:
		tdma->cur_mode = tdma->write | 0x10;   // turn on auto init
		break;
	}

	outportb (tdma->single, tdma->dma_disable);          /* disable chan   */
	outportb (tdma->mode, tdma->cur_mode);               /* set mode       */
	outportb (tdma->clear_ff, 0);                        /* clear f/f      */
	outportb (tdma->addr, tdma->cur_addr & 0xff);        /* LSB            */
	outportb (tdma->addr, (tdma->cur_addr >> 8) & 0xff); /* MSB            */
	outportb (tdma->page,tdma->cur_page);                /* page #         */
	outportb (tdma->clear_ff, 0);                        /* clear f/f      */
	outportb (tdma->count, tcount & 0x0ff);              /* LSB count      */
	outportb (tdma->count, (tcount >> 8) & 0x0ff);       /* MSB count      */
	outportb (tdma->single, tdma->dma_enable);           /* enable         */

	return ULTRA_OK;
}


void UltraStopRecordDma (void)
{
	DMA_ENTRY *tdma;
	uint8_t val;

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, SAMPLE_CONTROL);
	val = inportb (_gf1_data.data_hi);
	val &= ~ENABLE_ADC;
	outportb (_gf1_data.data_hi, val);

	// Disable channel
	tdma  = &_gf1_dma[_gf1_data.adc_dma_chan - 1];
	outportb (tdma->single, tdma->dma_disable);
	tdma->flags &= ~DMA_PENDING;

	// Clear flag that irq handler clears when the xfer is complete
	_gf1_data.flags &= ~ADC_DMA_BUSY;

	LEAVE_CRITICAL;
}

void UltraStopPlayDma (void)
{
	DMA_ENTRY *tdma;
	uint8_t val;

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, DMA_CONTROL);
	val = inportb (_gf1_data.data_hi);
	val &= ~DMA_ENABLE;
	outportb (_gf1_data.data_hi, val);

	// Disable channel
	tdma  = &_gf1_dma[_gf1_data.dram_dma_chan - 1];
	outportb (tdma->single, tdma->dma_disable);
	tdma->flags &= ~DMA_PENDING;

	// Clear flag that irq handler clears when the xfer is complete
	_gf1_data.flags &= ~DRAM_DMA_BUSY;

	LEAVE_CRITICAL;
}


int UltraDramDmaBusy (void)
{
	return _gf1_dma[_gf1_data.dram_dma_chan - 1].flags & DMA_PENDING;
}


void UltraWaitDramDma (void)
{
	_gf1_data.flags &= ~DRAM_DMA_NOWAIT;

	while (_gf1_data.flags & DRAM_DMA_BUSY);
}


/*
 * This function will prime the pc's dma controller for sending to
 * or receiving data from the UltraSound. The last thing it does is
 * start the transfer. It either wait for it to complete or it
 * could return immediately ...
 *
 * NOTE: This example will NOT attempt to explain how the pc's
 *       dma controller works. If you want to know more than you
 *       can by looking at this code, there are lots of documents
 *       available that go into great detail.
 */
int UltraDmaDram_64K(void *pc_ptr, uint32_t size, uint32_t ultra_ptr, uint8_t control, int wait)
{
	DMA_ENTRY *tdma;
	uint32_t dram_address;
	int type;
	int ret;

	tdma = &_gf1_dma[_gf1_data.dram_dma_chan - 1];

	if (control & DMA_READ)
	{
		if (control & DMA_AUTO_INIT)
			type = INDEF_READ;
		else
			type = READ_DMA;
	}
	else
	{
		if (control & DMA_AUTO_INIT)
			type = INDEF_WRITE;
		else
			type = WRITE_DMA;
	}

	control &= ~DMA_AUTO_INIT;   // turn this off now

	ret = PrimeDma (pc_ptr, type, size, _gf1_data.dram_dma_chan);
	if (ret != ULTRA_OK)
		return ret;

	// If its a 16 bit dma channel, an extra translation is necessary
	if (_gf1_data.dram_dma_chan >= 4)
		dram_address = convert_to_16bit (ultra_ptr);
	else
		dram_address = ultra_ptr;

	/*
	 * Only use the upper 16 bits. This means that an 8 bit xfer MUST
	 * start on a 16 byte boundary and a 16 bit xfer MUST start on a 32
	 * byte boundary.
	 */

	dram_address >>= 4;

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, SET_DMA_ADDRESS);
	outportw (_gf1_data.data_low, dram_address);

	LEAVE_CRITICAL;

	_gf1_data.flags |= DRAM_DMA_BUSY;

	tdma->cur_control = control;
	UltraStartDramDma (control);

	if (wait)
		UltraWaitDramDma ();
	else
		_gf1_data.flags |= DRAM_DMA_NOWAIT;

	return ULTRA_OK;
}


/*
 * This function will download/upload up to 64K of data into/out the
 * dram on the ultrasound card. Since the card cannot DMA
 * across 256K boundaries, this routine will split up the xfer ....
 */
static int UltraDmaDram (void *pc_ptr, uint32_t size, uint32_t ultra_ptr, uint8_t control, int wait)
{
	uint32_t end;
	uint32_t start_page, end_page;
	uint32_t start_size = 0;
	void *nxt_pc_ptr;
	uint32_t nxt_ptr = 0;
	uint32_t nxt_size = 0;
	int two_flag = 0;
	int ret;

	end = ultra_ptr + size - 1;

	start_page = ultra_ptr >> 18;   // isolate the 256K page
	end_page   = end >> 18;         // isolate the 256K page

	if (start_page != end_page)    // crossed page boundary
	{
		nxt_ptr = end_page << 18;      /* where second part goes in DRAM */
		nxt_size = end - nxt_ptr;      /* How much to put there          */
		start_size = size - nxt_size;  /* How much in first page of DRAM */
					 /* where in PC RAM is second part */
		nxt_pc_ptr = (void *)((char *) pc_ptr + start_size);
		two_flag = 1;
	}
	else
		start_size = size;

	ret = UltraDmaDram_64K (pc_ptr, start_size, ultra_ptr, control, wait);

	if (ret != ULTRA_OK)
		return ret;

	// Now send second half - probably SHOULD put this in IRQ handler

	if (two_flag)
	{
		if (!wait)
			UltraWaitDramDma ();

		ret = UltraDmaDram_64K (nxt_pc_ptr, nxt_size, nxt_ptr, control, wait);

		if (ret != ULTRA_OK)
			return ret;
	}

	return ULTRA_OK;
}


int UltraDownload (void *dataptr, uint8_t control, uint32_t dram_loc, uint32_t len, int wait)
{
	control &= ~DMA_READ;

	return UltraDmaDram (dataptr, len, dram_loc, control, wait);
}


int UltraUpload (void *dataptr, uint8_t control, uint32_t dram_loc, uint32_t len, int wait)
{
	control |= DMA_READ;

	return UltraDmaDram (dataptr, len, dram_loc, control, wait);
}


int UltraRecordDmaBusy (void)
{
	return _gf1_dma[_gf1_data.adc_dma_chan - 1].flags & DMA_PENDING;
}


void UltraWaitRecordDma (void)
{
	_gf1_data.flags &= ~ADC_DMA_NOWAIT;

	while (_gf1_data.flags & ADC_DMA_BUSY);
}


void UltraSetRecordFrequency (uint32_t rate)
{
	/*
	 * First calculate as if its for a record using the formula:
	 *      rate = CLOCK_RATE / (16*(ADRS+2))
	 */
	uint8_t adsr = ((CLOCK_RATE >> 4) / rate) - 2;

	ENTER_CRITICAL;

	outportb (_gf1_data.reg_select, SET_SAMPLE_RATE);
	outportb (_gf1_data.data_hi, adsr);
     
	LEAVE_CRITICAL;
}


int UltraPrimeRecord (void *pc_ptr, uint32_t size, int repeat)
{
	int mode;

	if (repeat)
		mode = INDEF_READ;
	else
		mode = READ_DMA;

	// Make sure the channel is not busy (recording or playback ... )
	return PrimeDma (pc_ptr,mode,size,_gf1_data.adc_dma_chan);
}


int UltraGoRecord (uint8_t control)
{
	DMA_ENTRY *tdma = &_gf1_dma[_gf1_data.adc_dma_chan - 1];

	_gf1_data.flags |= ADC_DMA_BUSY;

	tdma->cur_control = control;

	UltraStartRecordDma (control);

	return ULTRA_OK;
}


int UltraRecordData (void *pc_ptr, uint8_t control, uint32_t size, int wait, int repeat)
{
	int ret;

	if ((ret = UltraPrimeRecord(pc_ptr, size, repeat)) != ULTRA_OK)
		return ret;

     UltraGoRecord (control);

     if (wait)
		 UltraWaitRecordDma ();
     else
		 _gf1_data.flags |= ADC_DMA_NOWAIT;

     return ULTRA_OK;
}


static uint32_t GetRecordDmaPos (int chan_num)
{
	DMA_ENTRY *tdma;
	uint32_t val1, val2;
	uint32_t low1, low2;
	uint32_t high1, high2;

	static unsigned int threshold[MAX_DMA + 1]
		= {30, 30, 30, 30, 30, 30, 30, 30};

	int i = 5;

	tdma = &_gf1_dma[chan_num - 1];

	ENTER_CRITICAL;

	if (tdma->flags & CALIB_COUNT)
	{
		/*
		 * This code is necessary to accomodate a virtualized DMA controller.
		 * If something (like emm386) virtualizes the DMA controller, we have
		 * to adjust our threshold point to accomodate the rollover problem
		 * in the DMA controller. The problem is when you read the low byte
		 * and the high byte rolls over before we get a chance to read it.
		 * A virtualized DMA controller aggravates the problem since it will
		 * take longer between reads ...
		 */

		tdma->flags &= ~CALIB_COUNT;

		while(i-- > 0)   // try up to 5 times, then use default
		{
			outportb (tdma->clear_ff, 0);

			low1  = inportb (tdma->count);
			high1 = inportb (tdma->count);
			low2  = inportb (tdma->count);
			high2 = inportb (tdma->count);

			if (high1 == high2)
			{
				threshold[chan_num] = (low1 - low2) / 2 + 2;

				break;
			}
		}
	}

	val2 = 1;   // first time thru condition ...
	
	while (1)
	{
		outportb (tdma->clear_ff, 0);

		low1  = inportb (tdma->count);
		high1 = inportb (tdma->count);
		val1  = (high1 << 8) + low1;

		// If count is not about to roll over, use this count
		// or if count equals 0xffff (dma complete) use this count
		if ((low1 > threshold[chan_num] && low1 != 0xff) || (val1 == 0xffff))
		{
			val2 = val1;
			break;
		}

		if (val2 == val1)
			break;

		val2 = val1;
	}

	LEAVE_CRITICAL;

	return val2 + 1;
}


uint32_t UltraReadRecordPosition (void)
{
	DMA_ENTRY *tdma;
	uint32_t actual_dma;
	uint32_t this_size;
	uint32_t total_size;

	tdma = &_gf1_dma[_gf1_data.adc_dma_chan - 1];

	actual_dma = GetRecordDmaPos(_gf1_data.adc_dma_chan);

	// Since it counts backwards, subtract this from size of transfer
	this_size = tdma->cur_size - actual_dma;

	// Now add in the amount sent (in case it crosses page)
	total_size = tdma->amnt_sent + this_size;

	if (_gf1_data.adc_dma_chan >= 4)
		total_size <<= 1;

	return total_size;
}
