
#include <stdint.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <stdio.h>
#include <conio.h>
#include <fcntl.h>

#include "gus.h"


#define LOGERR(fmt, ...) fprintf (stderr, "\nERROR: " fmt "\n\n", ## __VA_ARGS__)

#define REC_SIZE (uint32_t) ((63UL * 1024UL))

static volatile uint32_t count1 = 0, count2 = 0;




static void __far HandleTimer1 (void) { count1++; return; }
static void __far HandleTimer2 (void) { count2++; return; }


int main (int argc, char *argv[])
{
	int voice_num = 0,
		timer_val,
		select = 0,
		dram_len = 0,
		done = 0;

#if 0
	_go32_dpmi_seginfo info;

	char filename[80], temp_char[10];

	uint8_t control, rec_control;

	void *data_ptr;

	int fd;

	long temp, select;

	uint32_t val, temp_val, dram_loc, begin, start_loop, end_loop,
		frequency,
		pos;
#endif


	ULTRA_CFG config;

	UltraGetCfg (&config);

	if (UltraProbe (config.base_port) == NO_ULTRA)
	{
		LOGERR("No card found");

		return -1;
	}

	if (UltraOpen (&config, 32) == NO_ULTRA)
	{
		LOGERR("Unable to open card");

		return -1;
	}

	// Grab the 80 microsecond timer handler
	UltraTimer1Handler (HandleTimer1);

	// Grab the 320 microsecond timer handler
	UltraTimer2Handler (HandleTimer2);

	printf ("\n"
	        "Example program for UltraSound Audio Card\n"
	        "=========================================\n");

	dram_len = UltraSizeDram ();

	printf ("Found %dK of DRAM\n\n", dram_len);

	while (!done)
	{
		printf ("0  - Exit\n");
		printf ("1  - Reset UltraSound\n");
//		printf ("2  - Download a sample to DRAM\n");
//		printf ("3  - Note on\n");
//		printf ("4  - Note off\n");
//		printf ("5  - Set Playback/Record rate\n");
//		printf ("6  - Set Balance\n");
//		printf ("7  - Set Volume\n");
//		printf ("8  - Set Channel to use\n");
//		printf ("9  - Read voice's absolute position\n");
//		printf ("10 - Ramp volume\n");
		printf ("11 - Demo timer #1 (80 usec)\n");
		printf ("12 - Demo timer #2 (320 usec)\n");
//		printf ("13 - Demo recording\n");

		while (1)
		{
			printf ("\nEnter Selection : ");

			if (scanf ("%d", &select))
				break;

			printf ("\nBad Selection\n");
			scanf ("%*s");
		}

		switch (select)
		{
		case 0: // Exit
			done = 1;
			break;

		case 1: // Reset Ultrasound

			UltraDisableOutput ();
			UltraReset (32);

			// Re-Init some voice parms to defaults ...
			UltraSetLinearVolume (voice_num, 256);
			UltraSetFrequency (voice_num, 22050);
			UltraSetBalance (voice_num, 7);
			UltraEnableOutput ();

			break;

#if 0
		case 2: // Download a sample to DRAM
			control = 0;

			printf ("Filename to get data from ? ");

			scanf ("%s",filename);

			if ((fd = open (filename, O_RDONLY | O_BINARY)) < 0)
			{
				printf ("Can't open file\n");

				break;
			}

			printf ("Amount to send down (bytes) ? ");
			scanf ("%lu", &val);
			data_ptr = malloc (val);

			if (data_ptr == NULL)
			{
				close (fd);
				break;
			}

			lseek (fd, 0, SEEK_SET);
			temp_val = read (fd, data_ptr, val);

			if (temp_val != val)
			{
				printf("Read failure %ld\n", temp_val);
				free(data_ptr);
				close(fd);
				break;
			}

			printf("Ultra DRAM location to put data ? ");
			scanf("%ld",&dram_loc);

			printf("8 bit (y/n) ? ");
			scanf("%s",temp_char);
			strupr(temp_char);
			if (strcmp(temp_char,"Y") == 0)
				control |= DMA_8;
			else
				control |= DMA_16;

			printf("Is it in twos compliment form ? ");
			scanf("%s",temp_char);
			strupr(temp_char);
			if (strcmp(temp_char,"Y") == 0)
				control &= ~DMA_CVT_2;
			else
				control |= DMA_CVT_2;

			info.size = (val + 15) / 16;
			if (_go32_dpmi_allocate_dos_memory (&info)) {
				printf ("BAD     %ld\n", info.size); exit(-1); }

			dosmemput (data_ptr, val, info.rm_segment * 16);

			printf ("memptr = %ld\n", (long) (info.rm_segment * 16));
			UltraDownload((void *)(info.rm_segment * 16), control, dram_loc, val, 1);

			_go32_dpmi_free_dos_memory (&info);
			free (data_ptr);
			close (fd);

			break;

		case 3: // Note on
			control = 0;

			printf("Begining location ? ");
			scanf("%ld", &begin);

			printf("Loop start location ? ");
			scanf("%ld", &start_loop);

			printf("End location ? ");
			scanf("%ld", &end_loop);

			printf("Loop type (0=none,1=uni-direct,2=bi_direct) ? ");
			scanf("%ld",&temp_val);
			if (temp_val > 2)
			{
				printf("Bad loop type\n");
				break;
			}

			switch (temp_val)
			{
			case 0: // no looping
				break;
			case 1: // uni-directional looping
				control |= 0x08;
				break;
			case 2: // bi-directional looping
				control |= 0x08;
				control |= 0x10;
				break;
			}

			printf("8 bit (y/n) ? ");
			scanf("%s",temp_char);
			strupr(temp_char);
			if (strcmp(temp_char,"Y") != 0)
			{
				control |= 0x04;
			}
			printf("Frequency (in hertz) = : ");
			scanf("%ld",&frequency);
			UltraVoiceOn (voice_num,begin,start_loop,end_loop,control,frequency);
			break;

		case 4: // Note off

			/*
			 * This could be used to change the end point and
			 * let the sample stop at end (sampled decay.... )
			 */
			// UltraSetVoiceEnd (voice_num,end_loop);

			UltraVoiceOff (voice_num,FALSE);
			break;

		case 5: // Set Playback/Record rate
			printf("New Frequency (in hertz) = : ");
			scanf("%ld",&frequency);
			printf("\n");

			UltraSetFrequency (voice_num,frequency);
			break;

		case 6: // Set Balance
			printf("New Balance = (0-15) : ");
			scanf("%ld",&val);
			printf("\n");

			UltraSetBalance (voice_num,val);
			break;

		case 7: // Set Volume
			printf("New Volume = (0-511) : ");
			scanf("%ld",&val);
			printf("\n");

			UltraSetLinearVolume (voice_num,val);
			break;

		case 8: // Set Channel to use
			printf("Channel to use ? ");
			scanf("%ld", &voice_num);
			break;

		case 9: // Read voice's absolute position
			printf("Voice #%ld position (hit a key to stop)\n",voice_num);
			while (!kbhit())
			{
				pos = UltraReadVoice (voice_num);
				printf("\r%ld ",pos);
			}
			getch();
			printf("\n\n");
			break;

		case 10: // Ramp volume
			printf("Start Volume = (0-511) : ");
			scanf("%ld",&val);
			printf("End Volume = (0-511) : ");
			scanf("%ld",&temp_val);
			printf("Milliseconds to Ramp Volume : ");
			scanf("%ld",&pos);
			printf("\n");
			UltraRampLinearVolume (voice_num,val,temp_val,pos,0x18);
			break;
#endif
		case 11: // Demo timer #1 (80 usec)
			count1 = 0;
			printf ("Time value (1-255) : ");
			scanf ("%d", &timer_val);
			printf ("Press any key to halt\n");
			UltraStartTimer (1, timer_val);
			while (!kbhit());
			UltraStopTimer (1);
			printf ("Count = %ld\n", count1);
			break;

		case 12: // Demo timer #2 (320 usec)
			count2 = 0;
			printf ("Time value (1-255) : ");
			scanf ("%d", &timer_val);
			printf ("Press any key to halt\n");
			UltraStartTimer (2, timer_val);
			while (!kbhit());
			UltraStopTimer (2);
			printf ("Count = %ld\n", count2);
			break;
#if 0
		case 13: // Demo recording

			info.size = (REC_SIZE) / 16;

			if (_go32_dpmi_allocate_dos_memory(&info))
			{
				printf("BAD     %ld\n", info.size);

				exit(-1);
			}

			rec_control = 0;
			control = 0;
			printf("Frequency (in hertz) = : ");
			scanf("%ld",&frequency);
			UltraSetRecordFrequency(frequency);
			UltraSetFrequency (voice_num,frequency);

			UltraEnableLineIn();
			UltraEnableMicIn(); 
			UltraEnableOutput();

			while(1)
			{
				printf("Hit a key to begin recording ....(ESC to quit)\n");
				while (!kbhit());

				if (getch() == 0x1b)
					break;

				UltraRecordData((void *)(info.rm_segment *16),rec_control,REC_SIZE,FALSE,FALSE);

				while (UltraRecordDmaBusy())
				{
					val = UltraReadRecordPosition();

					printf("%05lu\r",val);
				}
				printf("\n");

				printf("Hit a key to begin playback ....(ESC to quit)\n");

				while (!kbhit());

				if (getch() == 0x1b)
					break;

				// start download with no wait. Then start voice.
				UltraDownload((void *)(info.rm_segment *16),control,0,REC_SIZE,1);
				UltraStartVoice (voice_num,0,0,REC_SIZE,0);

				/*
				 * If this check is ommitted, it will begin recording
				 * while it is still downloading & playing ... Try it ...
				 */

				while (!UltraVoiceStopped (voice_num))
				{
					pos = UltraReadVoice (voice_num);
					printf("%05ld\r",pos);
				}
				printf("\n");
			}

			UltraDisableLineIn();
			UltraDisableMicIn();
			_go32_dpmi_free_dos_memory(&info);

			break;
#endif
		default:
			break;
		}
	}

	UltraClose ();

	return 0;
}
