/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#ifndef __GF1HWARE_H__
#define __GF1HWARE_H__

#include <stdint.h>


// Make GF1 address for direct chip i/o.
#define ADDR_HIGH(x) ((unsigned int)((unsigned int)((x>>7L)&0x1fffL)))
#define ADDR_LOW(x)  ((unsigned int)((unsigned int)((x&0x7fL)<<9L)))

#define JOYSTICK_TIMER  0x201           /* 201 */
#define JOYSTICK_DATA   0x201           /* 201 */

#define GF1_MIDI_CTRL   0x100           /* 3X0 MIDI Control/Status - (GMCR)16-2 & (GMSR)16-3 */
#define GF1_MIDI_DATA   0x101           /* 3X1 MIDI TX/RX - (GMTDR/GMRDR) 16-4 */

#define GF1_PAGE        0x102           /* 3X2 Sync Voice Select (SVSR) 14-1 */
#define GF1_REG_SELECT  0x103           /* 3X3 General Index (IGIDXR) 12-12 */
#define GF1_DATA_LOW    0x104           /* 3X4 16-bit data port (I16DP) 12-12 */
#define GF1_DATA_HI     0x105           /* 3X5 8-bit data port (I8DP) 12-12 */
#define GF1_IRQ_STAT    0x006           /* 2X6 IRQ status (U2X6R) 12-3 */
#define GF1_DRAM        0x107           /* 3X7 LMC Byte Data (LMBDR) 15-1 */

#define GF1_MIX_CTRL    0x000           /* 2X0 Mix Control (UMCR) 12-1 */

// These are not ideal.  They are adlib emulation defaults.
#define GF1_TIMER_CTRL  0x008           /* 2X8 UACWR/UASRR (12-3/12-13) */
#define GF1_TIMER_DATA  0x009           /* 2X9 UADR (12-4/12-13) */
#define GF1_IRQ_CTRL    0x00B           /* 2XB UICI - Only when UMCR[6]=1,URCRP[2:0]=0 (12-9)*/

// The GF1 Hardware clock
#define CLOCK_RATE              9878400L

// Mixer control bits
#define ENABLE_LINE             0x01
#define ENABLE_DAC              0x02
#define ENABLE_MIC              0x04

// interrupt controller 1
#define CNTRL_8259              0x21
#define OCR_8259                0x20
#define EOI                     0x20
#define REARM3                  0x2F3
#define REARM5                  0x2F5

// interrupt controller 2
#define CNTRL_M_8259            0x21
#define CNTRL_M2_8259           0xA1
#define OCR_2_8259              0xA0

#define DMA_CONTROL             0x41 // [b] LDMACI 15-1
#define SET_DMA_ADDRESS         0x42 // [w] LDSALI 15-2
#define SET_DRAM_LOW            0x43 // [w] LMALI  15-3
#define SET_DRAM_HIGH           0x44 // [b] LMAHI  15-3

#define TIMER_CONTROL           0x45 // [b] UASBCI 12-12
#define TIMER1                  0x46 // [b] UATI1  12-13
#define TIMER2                  0x47 // [b] UATI2  12-13

#define SET_SAMPLE_RATE         0x48 // Not in InterWave manual
#define SAMPLE_CONTROL          0x49 // [b] USCI   12-13

#define SET_JOYSTICK            0x4B // [b] GJTDI  16-1
#define MASTER_RESET            0x4C // [b] URSTI  12-4

// Voice register mapping.
#define SET_CONTROL             0x00 // [w] SACI   14-8
#define SET_FREQUENCY           0x01 // [w] SFCI   14-7
#define SET_START_HIGH          0x02 // [w] SASHI  14-4
#define SET_START_LOW           0x03 // [w] SASLI  14-5
#define SET_END_HIGH            0x04 // [w] SAEHI  14-5
#define SET_END_LOW             0x05 // [w] SAELI  14-5
#define SET_VOLUME_RATE         0x06 // [b] SVRI   14-10
#define SET_VOLUME_START        0x07 // [b] SVSI   14-9
#define SET_VOLUME_END          0x08 // [b] SVEI   14-9
#define SET_VOLUME              0x09 // [w] SVLI   14-10
#define SET_ACC_HIGH            0x0a // [w] SAHI   14-6
#define SET_ACC_LOW             0x0b // [w] SALI   14-6
#define SET_BALANCE             0x0c // [w] SROI   14-12
#define SET_VOLUME_CONTROL      0x0d // [b] SVCI   14-10
#define SET_VOICES              0x0e // [b] SAVI   14-1

#define GET_CONTROL             0x80
#define GET_FREQUENCY           0x81
#define GET_START_HIGH          0x82
#define GET_START_LOW           0x83
#define GET_END_HIGH            0x84
#define GET_END_LOW             0x85
#define GET_VOLUME_RATE         0x86
#define GET_VOLUME_START        0x87
#define GET_VOLUME_END          0x88
#define GET_VOLUME              0x89
#define GET_ACC_HIGH            0x8a
#define GET_ACC_LOW             0x8b
#define GET_BALANCE             0x8c
#define GET_VOLUME_CONTROL      0x8d
#define GET_VOICES              0x8e
#define GET_IRQV                0x8f // [b] SVII   14-2







/*

Mnem.   Description                         per  r/w  reg  idx    deflt  Page
------------------------------------------------------------------------------
UMCR    Mix Control                      *   1x  r/w  2x0   ---   --03h  12-1
UISR    IRQ Status                       *   1x  r/   2x6   ---   --00h  12-2
UACWR   AdLib Command Write              *   1x   /w  2x8   ---   --00h  12-3
UASRR   AdLib Status Read                *   1x  r/   2x8   ---   --00h  12-3
UADR    AdLib Data                       *   1x  r/w  2x9   xxx   --00h  12-4
          - Enable waveform control                   2x9    1    --??h
          - Timer #1 data                             2x9    2    --??h
          - Timer #2 data                             2x9    3    --??h
          - Timer control flags                       2x9    4    --??h
          - Speech synthesis mode                     2x9    5    --??h
          - Amplitude Modulation / Vibrato            2x9  20-35  --??h
          - Level key scaling / Total level           2x9  40-55  --??h
          - Attack / Decay rate                       2x9  60-75  --??h
          - Sustain / Release rate                    2x9  80-95  --??h
          - Octave / Frequency Number                 2x9  a0-b8  --??h
          - Feedback / Algorithm                      2x9  c0-c8  --??h
          - Waveform Selection                        2x9  e0-f5  --??h
UACRR   AdLib Command Read (PnP Only)        1x  r/   2xa   ---   --00h  12-3
UASWR   AdLib Status Write (PnP Only)        1x   /w  2xa   ---   --00h  12-3
UHRDP   GUS Hidden Register Data Port        1x  r/w  2xb   --—   --00h  12-5
UDCI      - DMA Channel Control              1x  r/w  2xb   -->   --00h  12-8  UMCR[6]=0, URCR[2:0]=0
UICI      - Interrupt Control            *   1x   /w  2xb   -->   --07h  12-9  UMCR[6]=1, URCR[2:0]=0
UGP1I     - General Purpose Register 1       1x  r/w  2xb   -->   --00h  12-10 URCR[2:0]=1
UGP2I     - General Purpose Register 2       1x  r/w  2xb   -->   --00h  12-10 URCR[2:0]=2
UGPA1I    - General Purpose Register 1 Addr  1x  r/w  2xb   -->   --00h  12-11 URCR[2:0]=3
UGPA2I    - General Purpose Register 2 Addr  1x  r/w  2xb   -->   --00h  12-11 URCR[2:0]=4
UCLRII    - Clear Interrupt                  1x   /w  2xb   -->   --xxh  12-11 URCR[2:0]=5
UJMPI     - Jumper                           1x  r/w  2xb   -->   --06h  12-11 URCR[2:0]=6
UI2XCR  Sound Blaster IRQ (PnP Only)         1x  r/w  2xc   ---   --00h  12-5
U2XCR   Sound Blaster 2XC (no IRQ) (PnP Only)1x   /w  2xd   --—   --00h  12-6
U2XER   Sound Blaster 2XE (PnP Only)         1x  r/w  2xe   ---   --00h  12-6
URCR    Register Control                     1x  r/w  2xf   ---   --00h  12-6 (Rev 3.4+)
USRR    Status Read (PnP Only ?)             1x  r/   2xf   ---   --01h  12-7
GMCR    MIDI Control                     *   1x  r/w  3x0   --—   --00h  16-2
GMSR    MIDI Status                      *   1x  r/   3x0   --—   --00h  16-3
GMTDR   MIDI Transmit Data               *   1x   /w  3x1   --—   --xxh  16-4
GMRDR   MIDI Receive Data                *   1x  r/   3x1   --—   --ffh  16-4
SVSR    Synth. Voice Select / GF1 Page   *   1x  r/w  3x2   --—   --00h  14-1
IGIDXR  General Index / Reg Select       *   1x  r/w  3x3   ---   --00h  12-12
I16DP   General 16-bit I/O Data Port     *   1x  r/w  3x4   xxx   xxxxh  12-12
I8DP    General 8-bit I/O Data Port      *   1x  r/w  3x5   xxx   --xxh  12-12
SACI      - Synth. Address Control       *   /v  r/w  3x5    0    --01h  14-8
SFCI      - Synth. Frequency Control     *   /v  r/w  3x4    1    0400h  14-7
SASHI     - Synth. Address Start High    *   /v  r/w  3x4    2    0000h  14-4
SASLI     - Synth. Address Start Low     *   /v  r/w  3x4    3    0000h  14-5
SAEHI     - Synth. Address End High      *   /v  r/w  3x4    4    0000h  14-5
SAELI     - Synth. Address End Low       *   /v  r/w  3x4    5    0000h  14-5
SVRI      - Synth. Volume Rate           *   /v  r/w  3x5    6    --00h  14-10
SVSI      - Synth. Volume Start          *   /v  r/w  3x5    7    --00h  14-9
SVEI      - Synth. Volume End            *   /v  r/w  3x5    8    --00h  14-9
SVLI      - Synth. Volume Level          *   /v  r/w  3x4    9    0000h  14-10
SAHI      - Synth. Address High          *   /v  r/w  3x4    A    0000h  14-6
SALI      - Synth. Address Low           *   /v  r/w  3x4    B    0000h  14-6
SROI      - Synth. Right Offset          *   /v  r/w  3x4    C    0700h  14-12
SVCI      - Synth. Volume Control        *   /v  r/w  3x5    D    --01h  14-10
SAVI      - Synth. Active Voices         *   1x  r/w  3x5    E    --CDh  14-1
SVII      - Synth. Voices IRQ            *   1x  r/   3x5    F    --E0h  14-2
LDMACI    - LMC DMA Control              *   1x  r/w  3x5   41    --00h  15-1
LDSALI    - LMC DMA Start Addr Low (19–4)*   1x  r/w  3x4   42    0000h  15-2
LMALI     - LMC I/O Addr Low (15–0)      *   1x   /w  3x4   43    0000h  15-3
LMAHI     - LMC I/O Addr High (23–16)    *   1x   /w  3x5   44    --00h  15-3
UASBCI    - AdLib–Sound Blaster Control  *   1x  r/w  3x5   45    --00h  12-12
UAT1I     - AdLib Timer 1                *   1x  r/w  3x5   46    --00h  12-13
UAT2I     - AdLib Timer 2                *   1x  r/w  3x5   47    --00h  12-13
          - Sampling Frequency           *   1x   /w  3x5   48    --??h
USCI      - ADC Sample Control           *   1x  r/w  3x5   49    --00h  12-13
GJTDI     - Joystick Trim DAC            *   1x  r/w  3x5   4B    --1Dh  16-1
URSTI     - GUS Reset                    *   1x   /w  3x5   4C    --00h  12-14
----------- AMD Interwave New Registers (+)
SUAI      + Synth. Upper Address             /v  r/w  3x5   10    --00h  14-4
SEAHI     + Synth. Effects Address High      /v  r/w  3x4   11    0000h  14-7
SEALI     + Synth. Effects Address Low       /v  r/w  3x4   12    0000h  14-7
SLOI      + Synth. Left Offset               /v  r/w  3x4   13    0000h  14-13
SEASI     + Synth. Effects Out Acc. Select   /v  r/w  3x5   14    --00h  14-14
SMSI      + Synth. Mode Select               /v  r/w  3x5   15    --02h  14-14
SEVI      + Synth. Effects Volume            /v  r/w  3x4   16    0000h  14-13
SFLFOI    + Synth. Frequency LFO             /v  r/w  3x5   17    --00h  14-8
SVLFOI    + Synth. Volume LFO                /v  r/w  3x5   18    --00h  14-11
SGMI      + Synth. Global Mode               1x  r/w  3x5   19    --00h  14-3
SLFOBI    + Synth. LFO Base Address          1x  r/w  3x4   1A    0000h  14-3
SROFI     + Synth. Right Offset Final        /v  r/w  3x4   1B    0700h  14-12
SLOFI     + Synth. Left Offset Final         /v  r/w  3x4   1C    0000h  14-13
SEVFI     + Synth. Effect Volume Final       /v  r/w  3x4   1D    0000h  14-14
SVIRI     + Synth. Voices IRQ Read           1x  r/   3x5   1F    --E0h  14-2
LDSAHI    + LMC DMA Start Addr Hi(23–20, 3–0)1x  r/w  3x5   50    --00h  15-3
LMSBAI    + LMC 16-Bit Access                1x  r/w  3x4   51    --xxh  15-4
LMCFI     + LMC Configuration                1x  r/w  3x4   52    0000h  15-4
LMCI      + LMC Control                      1x  r/w  3x5   53    --00h  15-5
LMRFAI    + LMC Record FIFO Base Addr(23–8)  1x  r/w  3x4   54    0000h  15-6
LMPFAI    + LMC Playback FIFO Base Addr(23–8)1x  r/w  3x4   55    0000h  15-6
LMFSI     + LMC FIFO Size                    1x  r/w  3x4   56    0000h  15-6
LDICI     + LMC DMA Interleave Control       1x  r/w  3x4   57    0000h  15-7
LDIBI     + LMC DMA Interleave BaseAddr(23–8)1x  r/w  3x4   58    0000h  15-7
ICMPTI    + Compatibility                    1x  r/w  3x5   59    --1Fh  12-15
IDECI     + Decode Control                   1x  r/w  3x5   5A    --7Fh  12-16
IVERI     + Version Number                   1x  r/w  3x5   5B    --20h  12-17
IEMUAI    + MPU-401 Emulation Control A      1x  r/w  3x5   5C    --00h  12-18
IEMUBI    + MPU-401 Emulation Control B      1x  r/w  3x5   5D    --30h  12-19
GMRFAI    + MIDI Receive FIFO Access         1x   /w  3x5   5E    --xxh  16-4
IEIRQI    + Emulation IRQ                    1x  r/w  3x5   60    --00h  12-20
LMBDR   LMC Byte Data                    *   1x  r/w  3x7   ---   --xxh  15-1

ICSMIX  Mixer Control Port               *   1x   /w  7x6
ICSMIX  Mixer Data Port                  *   1x   /w  3x6

MAXCTRL UltraMax Control Port (MAX ONLY)     1x   /w  3x6
 */






// MIDI defines

#define MIDI_RESET              0x03
#define MIDI_ENABLE_XMIT        0x20
#define MIDI_ENABLE_RCV         0x80

#define MIDI_RCV_FULL           0x01
#define MIDI_XMIT_EMPTY         0x02
#define MIDI_FRAME_ERR          0x10
#define MIDI_OVERRUN            0x20
#define MIDI_IRQ_PEND           0x80


// JOYSTICK defines

#define JOY_POSITION            0x0f
#define JOY_BUTTONS             0xf0


// GF1 irq/dma programmable latches

// GF1_IRQ_STATUS (port 3X6)
#define MIDI_TX_IRQ             0x01            /* pending MIDI xmit IRQ   */
#define MIDI_RX_IRQ             0x02            /* pending MIDI recv IRQ   */
#define GF1_TIMER1_IRQ          0x04            /* general purpose timer   */
#define GF1_TIMER2_IRQ          0x08            /* general purpose timer   */
#define WAVETABLE_IRQ           0x20            /* pending wavetable IRQ   */
#define ENVELOPE_IRQ            0x40            /* pending volume envelope IRQ */
#define DMA_TC_IRQ              0x80            /* pending dma tc IRQ      */


// GF1_MIX_CTRL (port 2X0)
#define ENABLE_LINE_IN          0x01            /* 0=enable                */
#define ENABLE_OUTPUT           0x02            /* 0=enable                */
#define ENABLE_MIC_IN           0x04            /* 1=enable                */
#define ENABLE_GF1_IRQ          0x08            /* 1=enable                */
#define GF122                   0x10            /* ??                      */
#define ENABLE_MIDI_LOOP        0x20            /* 1=enable loop back      */
#define SELECT_GF1_REG          0x40            /* 0=irq latches           */
                                                /* 1=dma latches           */

// GF1 global registers ($41-$4C)

// DMA control register
#define DMA_ENABLE              0x01
#define DMA_READ                0x02     /* 1=read,0=write          */
#define DMA_WIDTH_16            0x04     /* 1=16 bit,0=8 bit (dma chan width) */
#define DMA_RATE                0x18     /* 00=fast, 11=slow        */
#define DMA_IRQ_ENABLE          0x20     /* 1=enable                */
#define DMA_IRQ_PENDING         0x40     /* read                    */
#define DMA_DATA_16             0x40     /* write (data width)      */
#define DMA_TWOS_COMP           0x80     /* 1=do twos comp          */

// These are the xfer rate bits ..
#define DMA_R0          0x00            /* Fastest DMA xfer (~650khz)      */
#define DMA_R1          0x08            /* fastest / 2                     */
#define DMA_R2          0x10            /* fastest / 4                     */
#define DMA_R3          0x18            /* Slowest DMA xfer (fastest / 8)  */

// SAMPLE control register
#define ENABLE_ADC              0x01
#define ADC_MODE                0x02            /* 0=mono, 1=stereo        */
#define ADC_DMA_WIDTH           0x04            /* 0=8 bit, 1=16 bit       */
#define ADC_IRQ_ENABLE          0x20            /* 1=enable                */
#define ADC_IRQ_PENDING         0x40            /* 1=irq pending           */
#define ADC_TWOS_COMP           0x80            /* 1=do twos comp          */

// RESET control register
#define GF1_MASTER_RESET        0x01            /* 0=hold in reset         */
#define GF1_OUTPUT_ENABLE       0x02            /* enable output           */
#define GF1_MASTER_IRQ          0x04            /* master IRQ enable       */


// GF1 voice specific registers ($00 - $0E and $80-$8f)

// ($0,$80) Voice control register
#define VOICE_STOPPED           0x01     // voice has stopped
#define STOP_VOICE              0x02     // stop voice
#define VC_DATA_TYPE            0x04     // 0=8 bit,1=16 bit
#define VC_LOOP_ENABLE          0x08     // 1=enable
#define VC_BI_LOOP              0x10     // 1=bi directional looping
#define VC_WAVE_IRQ             0x20     // 1=enable voice wave irq
#define VC_DIRECT               0x40     // 0=increasing,1=decreasing
#define VC_IRQ_PENDING          0x80     // 1=wavetable irq pending

/* ($1,$81) Frequency control      */
/* Bit 0  - Unused                 */
/* Bits 1-9 - Fractional portion   */
/* Bits 10-15 - Integer portion    */

/* ($2,$82) Accumulator start address (high)  */
/* Bits 0-11 - HIGH 12 bits of address        */
/* Bits 12-15 - Unused                        */

/* ($3,$83) Accumulator start address (low)   */
/* Bits 0-4 - Unused                          */
/* Bits 5-8 - Fractional portion              */
/* Bits 9-15 - Low 7 bits of integer portion  */

/* ($4,$84) Accumulator end address (high)    */
/* Bits 0-11 - HIGH 12 bits of address        */
/* Bits 12-15 - Unused                        */

/* ($5,$85) Accumulator end address (low)     */
/* Bits 0-4 - Unused                          */
/* Bits 5-8 - Fractional portion              */
/* Bits 9-15 - Low 7 bits of integer portion  */


/* ($6,$86) Volume Envelope control register  */
#define VL_RATE_MANTISSA                0x3f
#define VL_RATE_RANGE                   0xC0

/* ($7,$87) Volume envelope start */
#define VL_START_MANT                   0x0F
#define VL_START_EXP                    0xF0

/* ($8,$88) Volume envelope end */
#define VL_END_MANT                     0x0F
#define VL_END_EXP                      0xF0

/* ($9,$89) Current volume register           */
/* Bits 0-3 are unused                        */
/* Bits 4-11 - Mantissa of current volume     */
/* Bits 10-15 - Exponent of current volume    */

/* ($A,$8A) Accumulator value (high) */
/* Bits 0-12 - HIGH 12 bits of current position (a19-a7) */

/* ($B,$8B) Accumulator value (low)           */
/* Bits 0-8 - Fractional portion              */
/* Bits 9-15 - Integer portion of low adress (a6-a0) */

/* ($C,$8C) Pan (balance) position */
/* Bits 0-3 - Balance position  0=full left, 0x0f=full right */

/* ($D,$8D) Volume control register */
#define VOLUME_STOPPED     0x01      // volume has stopped
#define STOP_VOLUME        0x02      // stop volume
#define VC_ROLLOVER        0x04      // Roll PAST end & gen IRQ
#define VL_LOOP_ENABLE     0x08      // 1=enable
#define VL_BI_LOOP         0x10      // 1=bi directional looping
#define VL_WAVE_IRQ        0x20      // 1=enable voice wave irq
#define VL_DIRECT          0x40      // 0=increasing,1=decreasing
#define VL_IRQ_PENDING     0x80      // 1=wavetable irq pending

/* ($E,$8E) # of Active voices       */
/* Bits 0-5 - # of active voices -1  */

/* ($F,$8F) - Sources of IRQs           */
/* Bits 0-4 - interrupting voice number */
/* Bit 5 - Always a 1                   */
#define VOICE_VOLUME_IRQ   0x40       // individual voice irq bit
#define VOICE_WAVE_IRQ     0x80       // individual waveform irq bit

#endif
