/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#include "common.h"


ULTRA_DATA _gf1_data;

/*
 * This function is used just to keep track of the version of the 
 * software a customer/developer has. It is not currently used
 * by the code at all....
 */
void UltraVersion (int *major, int *minor)
{
	*major = 2;
	*minor = 11;
}


int UltraOpen (ULTRA_CFG *config, int voices)
{
	int temp;

	memset (&_gf1_data, 0, sizeof (_gf1_data));

	_gf1_data.base_port     = config->base_port;
	_gf1_data.dram_dma_chan = config->dram_dma_chan;
	_gf1_data.adc_dma_chan  = config->adc_dma_chan;
	_gf1_data.gf1_irq_num   = config->gf1_irq_num;
	_gf1_data.midi_irq_num  = config->midi_irq_num;

	_gf1_data.mix_image     = 0x0b;
	_gf1_data.voices        = voices;
	_gf1_data.reserved_dram = 32;

	if ((temp = UltraProbe (_gf1_data.base_port)) == NO_ULTRA)
		return temp;

	UltraDisableLineIn ();
	UltraDisableMicIn ();
	UltraDisableOutput ();

	if ((temp = UltraReset (voices)) != ULTRA_OK)
		return temp;

	UltraSetInterface (_gf1_data.dram_dma_chan, _gf1_data.adc_dma_chan,
	                   _gf1_data.gf1_irq_num, _gf1_data.midi_irq_num);

	SetIrqHandlers (_gf1_data.gf1_irq_num, _gf1_data.midi_irq_num);

	SetIrqs (_gf1_data.gf1_irq_num, _gf1_data.midi_irq_num);

	UltraEnableOutput ();

	UltraMemInit ();

	return ULTRA_OK;
}


int UltraClose (void)
{
	UltraDisableOutput ();
	UltraDisableLineIn ();
	UltraDisableMicIn ();
	UltraReset (14);

	ResetIrqs (_gf1_data.gf1_irq_num, _gf1_data.midi_irq_num);
	ReSetIrqHandlers (_gf1_data.gf1_irq_num, _gf1_data.midi_irq_num);

	return ULTRA_OK;
}


/*
 * This function will parse the configuration variable "ULTRASND"
 * to find the settings for the configuration of the board.
 */
void UltraGetCfg (ULTRA_CFG *config)
{
	char *ptr;

	// defaults
	config->base_port     = 0x220;
	config->dram_dma_chan = 1;
	config->adc_dma_chan  = 1;
	config->gf1_irq_num   = 11;
	config->midi_irq_num  = 5;

	if ((ptr = getenv ("ULTRASND")))
	{
		sscanf (ptr,"%x,%d,%d,%d,%d",
		        &config->base_port,
		        &config->dram_dma_chan,
		        &config->adc_dma_chan,
		        &config->gf1_irq_num,
		        &config->midi_irq_num);
	}

	return;
}


/*
 * This function performs a full reset & initialization on the
 * UltraSound card.
 */
int UltraReset (int voices)
{
	register int v;

	uint16_t select   = _gf1_data.reg_select;
	uint16_t data_low = _gf1_data.data_low;
	uint16_t data_hi  = _gf1_data.data_hi;

	if ((voices < 14) || (voices > 32))
		return BAD_NUM_OF_VOICES;

	_gf1_data.voices = voices;
	_gf1_data.timer_ctrl = 0;
	_gf1_data.timer_mask = 0;

	/*
	 * Set voice target to zero so it does not get summed in for
	 * voices that are not running. If their volumes are not at
	 * zero, whatever value they are pointing at, will get summed
	 * into the output. By setting that location to 0, that voice
	 * will have no contribution to the output (2 locations are
	 * done in case voice is set to 16 bits ... )
	 */

	UltraPokeData (_gf1_data.base_port, 0, 0);
	UltraPokeData (_gf1_data.base_port, 1, 0);

	ENTER_CRITICAL;

	// Pull a reset on the GF1
	outportb (select, MASTER_RESET);
	outportb (data_hi, 0x00);

	for (v = 0; v < 10; v++)
		gf1_delay ();

	outportb (select, MASTER_RESET);
	outportb (data_hi, GF1_MASTER_RESET);

	for (v = 0; v < 10; v++)
		gf1_delay ();

	outportb (_gf1_data.midi_control, MIDI_RESET);

	for (v = 0; v < 10; v++)
		gf1_delay ();

	outportb (_gf1_data.midi_control, 0x00);

	// Clear all interrupts.
	outportb (select, DMA_CONTROL);
	outportb (data_hi, 0x00);
	outportb (select, TIMER_CONTROL);
	outportb (data_hi, 0x00);
	outportb (select, SAMPLE_CONTROL);
	outportb (data_hi, 0x00);

	outportb (select, SET_VOICES);
	outportb (data_hi, (voices - 1) | 0xC0);

	// Reading the status ports will clear the irqs.
	inportb (_gf1_data.irq_status);
	outportb (select, DMA_CONTROL);
	inportb (data_hi);
	outportb (select, SAMPLE_CONTROL);
	inportb (data_hi);
	outportb (select, GET_IRQV);
	inportb (data_hi);

	for (v = 0; v < voices; v++)
	{
		outportb (_gf1_data.voice_select, v);

		outportb (select, SET_CONTROL);   // Stop the voice and volume
		outportb (data_hi, VOICE_STOPPED | STOP_VOICE);
		outportb (select, SET_VOLUME_CONTROL);
		outportb (data_hi, VOLUME_STOPPED | STOP_VOLUME);

		gf1_delay ();

		/*
		 * Initialize each voice specific registers. This is not
		 * really necessary, but is nice for completeness sake.
		 * Each application will set up these to whatever values
		 * it needs.
		 */
		outportb (select, SET_FREQUENCY);
		outportw (data_low, 0x0400);
		outportb (select, SET_START_HIGH);
		outportw (data_low, 0);
		outportb (select, SET_START_LOW);
		outportw (data_low, 0);
		outportb (select, SET_END_HIGH);
		outportw (data_low, 0);
		outportb (select, SET_END_LOW);
		outportw (data_low, 0);
		outportb (select, SET_VOLUME_RATE);
		outportb (data_hi, 0x01);
		outportb (select, SET_VOLUME_START);
		outportb (data_hi, 0x10);
		outportb (select, SET_VOLUME_END);
		outportb (data_hi, 0xe0);
		outportb (select, SET_VOLUME);
		outportw (data_low, 0x0000);

		outportb (select, SET_ACC_HIGH);
		outportw (data_low, 0);
		outportb (select, SET_ACC_LOW);
		outportw (data_low, 0);
		outportb (select, SET_BALANCE);
		outportb (data_hi, 0x07);
	}

	inportb (_gf1_data.irq_status);
	outportb (select, DMA_CONTROL);
	inportb (data_hi);
	outportb (select, SAMPLE_CONTROL);
	inportb (data_hi);
	outportb (select, GET_IRQV);
	inportb (data_hi);

	/* Set up GF1 Chip for interrupts & enable DACs. */
	outportb (select, MASTER_RESET);
	outportb (data_hi, GF1_MASTER_RESET | GF1_OUTPUT_ENABLE | GF1_MASTER_IRQ);

	LEAVE_CRITICAL;

	return ULTRA_OK;
}


/*
 * This function returns the # of K found on the card
 */
int UltraSizeDram (void)
{
	int i;

	uint8_t save0 = UltraPeekData (_gf1_data.base_port, 0);

	UltraPokeData (_gf1_data.base_port, 0, 0xaa);

	if (UltraPeekData (_gf1_data.base_port, 0) != 0xaa)
		return 0;

	UltraPokeData (_gf1_data.base_port, 0, 0);

	for (i = 1; i < 1024; i++)
	{
		uint32_t loc;
		uint8_t save1, val = UltraPeekData (_gf1_data.base_port, 0);

		if (val != 0) break;

		loc = i << 10;

		save1 = UltraPeekData (_gf1_data.base_port, loc);

		UltraPokeData (_gf1_data.base_port, loc, 0xaa);

		if (UltraPeekData (_gf1_data.base_port, loc) != 0xaa)
			break;

		UltraPokeData (_gf1_data.base_port, loc, save1);
	}

	UltraPokeData (_gf1_data.base_port, 0, save0);

	return i;
}


int UltraPing (uint16_t port)
{
	uint8_t val0, save_val0 = UltraPeekData (port, 0);
	uint8_t val1, save_val1 = UltraPeekData (port, 1);

	UltraPokeData (port, 0, 0xAA);
	UltraPokeData (port, 1, 0x55);

	val0 = UltraPeekData (port, 0);
	val1 = UltraPeekData (port, 1);

	UltraPokeData (port, 0, save_val0);
	UltraPokeData (port, 1, save_val1);

	if ((val0 == 0xAA) && (val1 == 0x55))
		return ULTRA_OK;

	return NO_ULTRA;
}


int UltraProbe (uint16_t port)
{
	_gf1_data.base_port     = port;
	_gf1_data.midi_data     = port + GF1_MIDI_DATA;
	_gf1_data.midi_control  = port + GF1_MIDI_CTRL;
	_gf1_data.voice_select  = port + GF1_PAGE;
	_gf1_data.reg_select    = port + GF1_REG_SELECT;
	_gf1_data.data_low      = port + GF1_DATA_LOW;
	_gf1_data.data_hi       = port + GF1_DATA_HI;
	_gf1_data.irq_status    = port + GF1_IRQ_STAT;
	_gf1_data.dram_data     = port + GF1_DRAM;
	_gf1_data.mix_control   = port + GF1_MIX_CTRL;
	_gf1_data.timer_control = port + GF1_TIMER_CTRL;
	_gf1_data.timer_data    = port + GF1_TIMER_DATA;
	_gf1_data.irq_control   = port + GF1_IRQ_CTRL;

	/* Pull a reset on the GF1 */
	outportb (_gf1_data.reg_select, MASTER_RESET);
	outportb (_gf1_data.data_hi, 0x00);

	/* Wait a little while ... */
	gf1_delay ();
	gf1_delay ();

	/* Release Reset */
	outportb (_gf1_data.reg_select, MASTER_RESET);
	outportb (_gf1_data.data_hi, GF1_MASTER_RESET);

	gf1_delay ();
	gf1_delay ();

	return UltraPing (port);
}


void UltraSetInterface (int dram, int adc, int gf1, int midi)
{
	uint8_t gf1_irq, midi_irq, dram_dma=0, adc_dma=0;
	uint8_t irq_control, dma_control;
	uint8_t mix_image;

	ENTER_CRITICAL;

	// Don't need to check for 0 irq #. Its latch entry = 0
	gf1_irq  = get_irq_latch (gf1);
	midi_irq = get_irq_latch (midi);
	midi_irq <<= 3;

	/* Set latch to 0 if dma channel is 0 */
	/* This takes this channel off line ... */

	if (dram != 0)
		dram_dma = get_dma_latch (dram);

	if (adc != 0)
		adc_dma = get_dma_latch (adc);

	adc_dma <<= 3;

	irq_control = dma_control = 0x0;

	mix_image = _gf1_data.mix_image;   // init image to everything disabled

	irq_control |= gf1_irq;
	if((gf1 == midi) && (gf1 != 0))
		irq_control |= 0x40;
	else
		irq_control |= midi_irq;

	dma_control |= dram_dma;
	if((dram == adc) && (dram != 0))
		dma_control |= 0x40;
	else
		dma_control |= adc_dma;

	// Set up for Digital ASIC
	outportb (_gf1_data.base_port+0x0f, 0x5);
	outportb (_gf1_data.mix_control, mix_image);
	outportb (_gf1_data.irq_control, 0x0);
	outportb (_gf1_data.base_port+0x0f, 0x0);

	// First do DMA control register
	outportb (_gf1_data.mix_control, mix_image);
	outportb (_gf1_data.irq_control, dma_control|0x80);

	// IRQ CONTROL REG
	outportb (_gf1_data.mix_control, mix_image|0x40);
	outportb (_gf1_data.irq_control, irq_control);

	// First do DMA control register
	outportb (_gf1_data.mix_control, mix_image);
	outportb (_gf1_data.irq_control, dma_control);

	// IRQ CONTROL REG
	outportb (_gf1_data.mix_control, mix_image|0x40);
	outportb (_gf1_data.irq_control, irq_control);

	// IRQ CONTROL, ENABLE IRQ
	// just to Lock out writes to irq\dma register ...
	outportb (_gf1_data.voice_select, 0);

	// enable output & irq, disable line & mic input
	mix_image |= 0x09;
	outportb (_gf1_data.mix_control, mix_image);

	// just to Lock out writes to irq\dma register ...
	outportb (_gf1_data.voice_select, 0);

	// put image back ....
	_gf1_data.mix_image = mix_image;

	LEAVE_CRITICAL;
}
