/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#ifndef __GUS_H__
#define __GUS_H__

#include <stdint.h>

typedef void (__far *MIDI_XMIT_CB)(uint8_t status);
typedef void (__far *MIDI_RECV_CB)(uint8_t status, uint8_t data);
typedef void (__far *GUS_TMR_CB)(void);
typedef void (__far *GUS_WAV_CB)(int voice);
typedef void (__far *GUS_VOL_CB)(int voice);
typedef void (__far *GUS_REC_CB)(void);
typedef void (__far *GUS_DMA_CB)(void);
typedef void (__far *GUS_AUX_CB)(void);

typedef enum
{
	ULTRA_OK = 0,
	BAD_NUM_OF_VOICES,
	NO_MEMORY,
	CORRUPT_MEM,
	NO_ULTRA,
	DMA_BUSY,
	BAD_DMA_ADDR,
	VOICE_OUT_OF_RANGE,
	VOICE_NOT_FREE,        // voice has already been allocated
	NO_FREE_VOICES,        // not any voices free
	NO_MIXER,              // ICS Mixer 2101 not present

} ULTRA_STATUS;


typedef struct _st_ultra_cfg
{
	int base_port;
	int dram_dma_chan;
	int adc_dma_chan;
	int gf1_irq_num;
	int midi_irq_num;

} ULTRA_CFG;

#define MIX_CHAN_0	0
#define MIX_CHAN_1	1
#define MIX_CHAN_2	2
#define MIX_CHAN_3	3
#define MIX_CHAN_4	4
#define MIX_CHAN_5	5

#define MIX_LEFT    0
#define MIX_RIGHT   1

// Mixer channels used on GUS - Channel #4 is NOT used
#define MIX_MIKE_IN	  MIX_CHAN_0
#define MIX_LINE_IN   MIX_CHAN_1
#define MIX_CD_IN     MIX_CHAN_2
#define MIX_GF1_OUT   MIX_CHAN_3
#define MIX_MASTER    MIX_CHAN_5

#define MIX_CTRL_LEFT    0x00
#define MIX_CTRL_RIGHT   0x01
#define MIX_ATTN_LEFT    0x02
#define MIX_ATTN_RIGHT   0x03
#define MIX_PAN          0x04

// gus.c
extern void UltraVersion (int *major, int *minor);
extern int UltraOpen (ULTRA_CFG *config, int voices);
extern int UltraClose (void);
extern void UltraGetCfg (ULTRA_CFG *config);
extern int UltraReset (int voices);
extern int UltraSizeDram (void);
extern int UltraPing (uint16_t port);
extern int UltraProbe (uint16_t port);
extern void UltraSetInterface (int dram, int adc, int gf1, int midi);

// irq.c
extern MIDI_XMIT_CB UltraMidiXmitHandler (MIDI_XMIT_CB handler);
extern MIDI_RECV_CB UltraMidiRecvHandler (MIDI_RECV_CB handler);
extern GUS_TMR_CB UltraTimer1Handler (GUS_TMR_CB handler);
extern GUS_TMR_CB UltraTimer2Handler (GUS_TMR_CB handler);
extern GUS_WAV_CB UltraWaveHandler (GUS_WAV_CB handler);
extern GUS_VOL_CB UltraVolumeHandler (GUS_VOL_CB handler);
extern GUS_REC_CB UltraRecordHandler (GUS_REC_CB handler);
extern GUS_DMA_CB UltraDramTcHandler (GUS_DMA_CB handler);
extern GUS_AUX_CB UltraAuxHandler (GUS_AUX_CB handler);

// icsmix.c
extern int UltraMixProbe (uint16_t);
extern void UltraMixAttn (int, int, int, int);
extern void UltraMixXparent (int);
extern void UltraMixMute (int, int);

// joy.c
extern void UltraTrimJoystick (uint8_t joyval);

// volume.c
extern void UltraRampVolume (int voice, int start, int end, uint8_t rate, uint8_t mode);
extern uint8_t UltraCalcRate (int start, int end, int mil_secs);
extern uint16_t UltraReadVolume (int voice);
extern void UltraVectorVolume (int voice, int end, uint8_t rate, uint8_t mode);
extern void UltraSetVolume (int voice, unsigned int volume);
extern void UltraStopVolume (int voice);
extern int UltraVolumeStopped (int voice);
extern void UltraSetLinearVolume (int voice, int index);
extern void UltraRampLinearVolume (int voice, int start_idx, int end_idx, int msecs, uint8_t mode);
extern void UltraVectorLinearVolume (int voice, int end_idx, uint8_t rate, uint8_t  mode);
extern void UltraSetBalance (int voice, int data);

// voice.c
extern void UltraClearVoices (void);
extern int UltraAllocVoice (int voice_num, int *new_num);
extern void UltraFreeVoice (int voice_num);
extern void UltraSetVoiceEnd (int voice, uint32_t end);
extern void UltraSetFrequency (int voice, uint32_t speed_khz);
extern void UltraSetLoopMode (int voice, uint8_t mode);
extern uint32_t UltraReadVoice (int voice);
extern void UltraSetVoice (int voice,uint32_t location);
extern uint8_t UltraPrimeVoice (int voice, uint32_t begin, uint32_t start, uint32_t end, uint8_t mode);
extern void UltraGoVoice (int voice, uint8_t mode);
extern void UltraStartVoice (int voice, uint32_t begin, uint32_t start, uint32_t end, uint8_t mode);
extern void UltraStopVoice (int voice);
extern int UltraVoiceStopped (int voice);
extern void UltraVoiceOn (int voice, uint32_t begin, uint32_t start_loop, uint32_t end_loop, uint8_t control, uint32_t freq);
extern void UltraVoiceOff (int voice, int end);

// memory.c
extern uint8_t UltraPeekData (uint16_t port, uint32_t address);
extern void UltraPokeData (uint16_t port, uint32_t address, uint8_t data);
extern void UltraPokeLong (uint32_t location, uint32_t value);
extern uint32_t UltraPeekLong (uint32_t location);
extern int UltraMemInit (void);
extern uint32_t UltraMaxAlloc (void);
extern int UltraMemAlloc (uint32_t size, uint32_t *location);
extern int UltraMemFree (uint32_t size, uint32_t location);

// mixer.c
extern int UltraGetOutput (void);
extern void UltraEnableOutput (void);
extern void UltraDisableOutput (void);
extern int UltraGetLineIn (void);
extern void UltraEnableLineIn (void);
extern void UltraDisableLineIn (void);
extern int UltraGetMicIn (void);
extern void UltraEnableMicIn (void);
extern void UltraDisableMicIn (void);

// dma.c
extern void UltraStartDramDma (uint8_t control);
extern void UltraStartRecordDma (uint8_t control);
extern void UltraStopRecordDma (void);
extern void UltraStopPlayDma (void);
extern int UltraDramDmaBusy (void);
extern void UltraWaitDramDma (void);
extern int UltraDmaDram_64K(void *pc_ptr, uint32_t size, uint32_t ultra_ptr, uint8_t control, int wait);
extern int UltraDownload (void *dataptr, uint8_t control, uint32_t dram_loc, uint32_t len, int wait);
extern int UltraUpload (void *dataptr, uint8_t control, uint32_t dram_loc, uint32_t len, int wait);
extern int UltraRecordDmaBusy (void);
extern void UltraWaitRecordDma (void);
extern void UltraSetRecordFrequency (uint32_t rate);
extern int UltraPrimeRecord (void *pc_ptr, uint32_t size, int repeat);
extern int UltraGoRecord (uint8_t control);
extern int UltraRecordData (void *pc_ptr, uint8_t control, uint32_t size, int wait, int repeat);
extern uint32_t UltraReadRecordPosition (void);

// midi.c
extern void UltraMidiXmit (uint8_t data);
extern uint8_t UltraMidiRecv (void);
extern uint8_t UltraMidiStatus (void);
extern void UltraMidiEnableRecv (void);
extern void UltraMidiEnableXmit (void);
extern void UltraMidiDisableRecv (void);
extern void UltraMidiDisableXmit (void);
extern void UltraMidiReset (void);

// timer.c
extern void UltraStartTimer (int timer, uint8_t time);
extern void UltraStopTimer (int timer);
extern int UltraTimerStopped (int timer);

#endif
