/*
 * Copyright (c) 1992, by FORTE
 *
 * This software is furnished under a license and may be used,
 * copied, or disclosed only in accordance with the terms of such
 * license and with the inclusion of the above copyright notice.
 * This software or any other copies thereof may not be provided or
 * otherwise made available to any other person. No title to and
 * ownership of the software is hereby transfered.
 */

#include "common.h"


/*
 * This function is used as a 1.6*3 microsecond (or longer) delay.
 * This is needed when trying to change any of the 'self-modifying
 * bits in the voice registers.
 */
void gf1_delay (void)
{
	int i;

	// pick a port and do 7 inb's to get a delay.  This needs to be
	//      altered to take advantage of delay.c from CC's game lib.

	for (i = 0; i < 7; i++)
		inportb (_gf1_data.dram_data);
}


/*
 * This function will convert the value read from the GF1 registers
 * back to a 'real' address.
 */

#define MAKE_MS_WORD(x)    ((uint32_t) ( (uint32_t) (x)) << 16)

uint32_t make_physical_address (uint32_t low, uint32_t high, uint8_t mode)
{
	uint32_t lower_16, upper_16, ret_address, bit_19_20;

	upper_16 = high >> 9;
	lower_16 = ((high & 0x01ff) << 7) | ((low >> 9) & 0x007f);

	ret_address = MAKE_MS_WORD(upper_16) + lower_16;

	if (mode & VC_DATA_TYPE)
	{
		bit_19_20 = ret_address & 0xC0000;
		ret_address <<= 1;
		ret_address  &= 0x3ffff;
		ret_address  |= bit_19_20;
	}
	
	return ret_address;
}


/*
 * This function will translate the address if the dma channel
 * is a 16 bit channel. This translation is not necessary for
 * an 8 bit dma channel.
 */
uint32_t convert_to_16bit (uint32_t address)
{
	uint32_t hold_address = address;

	address = address >> 1;          /* Convert to 16 translated address. */

	address &= 0x0001ffffL;          /* Zero out bit 17.                  */

	address |= (hold_address & 0x000c0000L);     /* Reset bits 18 and 19. */

	return address;
}
